"""Classes that allow interacting with specific Agent devices."""

import logging
from enum import Enum
from typing import Optional
import asyncio
from urllib.parse import urlencode

_LOGGER = logging.getLogger(__name__)

class TimePeriod(Enum):
	"""Represents a period of time to check for events."""

	@property
	def period(self) -> str:
		"""Get the period of time."""
		# pylint: disable=unsubscriptable-object
		return self.value[0]

	@property
	def title(self) -> str:
		"""Explains what is measured in this period."""
		# pylint: disable=unsubscriptable-object
		return self.value[1]

	@staticmethod
	def get_time_period(value):
		"""Get the corresponding TimePeriod from the value.

		Example values: 'all', 'hour', 'day', 'week', or 'month'.
		"""
		for time_period in TimePeriod:
			if time_period.period == value:
				return time_period
		raise ValueError('{} is not a valid TimePeriod'.format(value))

	ALL = ('all', 'Events')
	HOUR = ('hour', 'Events Last Hour')
	DAY = ('day', 'Events Last Day')
	WEEK = ('week', 'Events Last Week')
	MONTH = ('month', 'Events Last Month')

class Device:
	"""Represents a device from Agent."""

	def __init__(self, client, raw_result):
		"""Create a new device."""
		self._client = client
		self._raw_result = raw_result
		self._oid = int(raw_result['id'])
		self._ot = int(raw_result['typeID'])

	@property
	def id(self) -> int:
		"""Get the Agent id number of this device."""
		# pylint: disable=invalid-name
		return self._oid

	@property
	def typeID(self) -> int:
		"""Get the Agent id number of this device."""
		# pylint: disable=invalid-name
		return self._ot

	@property
	def client(self):
		"""Get the Agent server client of this device."""
		# pylint: disable=invalid-name
		return self._client

	@property
	def name(self) -> str:
		"""Get the name of this device."""
		return self._raw_result['name']


	async def update(self):
		"""Update the device from the Agent server."""
		state = await self._client.get_state('command.cgi?cmd=getObject&oid={0}&ot={1}'.format(self._oid,self._ot))
		if state is not None:
			self._raw_result = state

	@property
	def mjpeg_image_url(self) -> str:
		"""Get the mjpeg url of this device."""
		return 'video.mjpg?oid={0}'.format(self._oid)
	
	@property
	def mp4_url(self) -> str:
		"""Get the mp4 video url of this device."""
		return 'video.mp4?oid={0}'.format(self._oid)
	
	@property
	def webm_url(self) -> str:
		"""Get the mp4 video url of this device."""
		return 'video.webm?oid={0}'.format(self._oid)

	@property
	def still_image_url(self) -> str:
		"""Get the still jpeg image url of this device."""
		return 'grab.jpg?oid={0}'.format(self._oid)

	@property
	def recording(self) -> bool:
		"""Indicate if this device is currently recording."""
		return self._raw_result['data']['recording']
	
	@property
	def alerted(self) -> bool:
		"""Indicate if this device has alerted."""
		return self._raw_result['data']['alerted']
	
	@property
	def detected(self) -> bool:
		"""Indicate if this device has detected."""
		return self._raw_result['data']['detected']

	@property
	def online(self) -> bool:
		"""Indicate if this device is currently online."""
		return self._raw_result['data']['online']

	@property
	def alerts_active(self) -> bool:
		"""Indicate if this device has alerts enabled."""
		return self._raw_result['data']['alertsActive']
	
	@property
	def detector_active(self) -> bool:
		"""Indicate if this device has alerts enabled."""
		return self._raw_result['data']['detectorActive']

	@property
	def connected(self) -> bool:
		"""Indicate if this device is currently connected."""
		return self._raw_result['data']['connected']

	@property
	def has_ptz(self) -> bool:
		"""Indicate if this device has PTZ capability."""
		return self._ot == 2 and self._raw_result['data']['ptzid'] != -1

	@property
	def width(self) -> int:
		"""Get the width of the device."""
		return self._raw_result['data']['width']

	@property
	def height(self) -> int:
		"""Get the height of the device."""
		return self._raw_result['data']['height']
		
	@property
	def mjpegStreamWidth(self) -> int:
		"""Get the computed stream width of the device."""
		if 'mjpegStreamWidth' in self._raw_result['data']:
			return self._raw_result['data']['mjpegStreamWidth']
		return 640

	@property
	def mjpegStreamHeight(self) -> int:
		"""Get the computed stream height of the device."""
		if 'mjpegStreamHeight' in self._raw_result['data']:
			return self._raw_result['data']['mjpegStreamHeight']
		return 480
		
	@property
	def location(self) -> str:
		"""Get the location of the device."""
		ind = self._raw_result['locationIndex']
		if ind > -1 and ind < len(self._client.locations):
			return self._client.locations[ind]['name']
		return ""
	
	@property
	def raw_result(self) -> dict:
		return self._raw_result


	async def enable(self):
		state = await self._client.get_state('command.cgi?cmd=switchOn&oid={0}&ot={1}'.format(self._oid, self._ot))
		if state is not None:
			self._raw_result['data']['online'] = True


	async def disable(self):
		state = await self._client.get_state('command.cgi?cmd=switchOff&oid={0}&ot={1}'.format(self._oid, self._ot))
		if state is not None:
			self._raw_result['data']['online'] = False


	async def record(self):
		state = await self._client.get_state('command.cgi?cmd=record&oid={0}&ot={1}'.format(self._oid, self._ot))
		if state is not None:
			self._raw_result['data']['recording'] = True


	async def record_stop(self):
		state = await self._client.get_state('command.cgi?cmd=recordStop&oid={0}&ot={1}'.format(self._oid, self._ot))
		if state is not None:
			self._raw_result['data']['recording'] = False


	async def alerts_on(self):
		state = await self._client.get_state('command.cgi?cmd=alertOn&oid={0}&ot={1}'.format(self._oid, self._ot))
		if state is not None:
			self._raw_result['data']['alertsActive'] = True


	async def alerts_off(self):
		state = await self._client.get_state('command.cgi?cmd=alertOff&oid={0}&ot={1}'.format(self._oid, self._ot))
		if state is not None:
			self._raw_result['data']['alertsActive'] = False

	async def detector_on(self):
		state = await self._client.get_state('command.cgi?cmd=detectorOn&oid={0}&ot={1}'.format(self._oid, self._ot))
		if state is not None:
			self._raw_result['data']['detectorActive'] = True


	async def detector_off(self):
		state = await self._client.get_state('command.cgi?cmd=detectorOff&oid={0}&ot={1}'.format(self._oid, self._ot))
		if state is not None:
			self._raw_result['data']['detectorActive'] = False

	async def snapshot(self):
		await self._client.get_state('command.cgi?cmd=snapshot&oid={0}&ot={1}'.format(self._oid, self._ot))


	async def get_events(self, time_period) -> Optional[int]:
		"""Get the number of events that have occurred on this device.

		Specifically only gets events that have occurred within the TimePeriod
		provided.
		"""
		date_filter = 3600*24*365*20
		if time_period == TimePeriod.HOUR:
			date_filter = 3600
		
		if time_period == TimePeriod.DAY:
			date_filter = 3600*24
		
		if time_period == TimePeriod.WEEK:
			date_filter = 3600*24*7
		
		if time_period == TimePeriod.MONTH:
			date_filter = 3600*24*30
		
		count_response = await self._client.get_state(
			'eventcounts.json?oid={0}&ot={1}&secs={2}'.format(self._oid, self._ot, date_filter)
		)
		if count_response is None:
			return 0

		return count_response['count']



"""Tests to verify the agent module."""

import unittest

from agent import a


class TestAgent(unittest.TestCase):
    """Tests to verify the Agent class."""

    def test_build_server_url_no_trailing_slash(self):
        """Verifies that server_url is correct with a trailing slash."""
        host = 'http://agent.com:8090'
        self.assertEqual(
            'http://agent.com/agent/',
            a.Agent._build_server_url(host)
        )



"""An API Client to interact with Agent."""
import logging
from typing import List, Optional
from urllib.parse import urljoin
from .exceptions import AgentConnectionError, AgentError

import aiohttp
import socket
import asyncio
import async_timeout

from agent.device import Device
from datetime import datetime

_LOGGER = logging.getLogger(__name__)


class Agent:
	"""The Agent DVR API client. Create one of these to begin."""

	DEFAULT_SERVER_PATH = '/'
	DEFAULT_TIMEOUT = 10
	OBJECTS_URL = 'command.cgi?cmd=getObjects'

	def __init__(self, server_host, session: aiohttp.client.ClientSession = None) -> None:
		"""Create an Agent API Client."""
		self._server_url = Agent._build_server_url(server_host)
		self._session = session or aiohttp.ClientSession()
		self._close_session = False
		if session is None:
			self._close_session = True
		self._conFailed = False 
		self.disconnected = datetime.now()
		self.reconnect_interval = 10
		self.profiles = []
		self.locations = []
		self.devices = []
		self._raw_result = {}
		return None

	async def close(self) -> None:
		if self._session and self._close_session:
			await self._session.close()

	async def get_state(self, api_url)  -> Optional[dict]:
		"""Perform a GET request on the specified Agent API URL."""
		return await self._agent_request('get', api_url)

	async def _agent_request(self, method, api_url,timeout=DEFAULT_TIMEOUT) -> dict:
		if self._conFailed:
			if (datetime.now()-self.disconnected).total_seconds() < self.reconnect_interval:
				return None
		headers = {
			"User-Agent": f"PythonAgentDVR/",
			"Accept": "application/json, text/plain, */*",
		}

		try:
			with async_timeout.timeout(timeout):
				response = await self._session.request(
					"GET", urljoin(self._server_url, api_url), json=None, headers=headers,
				)
				response.raise_for_status()
		except asyncio.TimeoutError as exception:
			self.disconnected = datetime.now()
			self._conFailed = True
			
			raise AgentConnectionError(
				"Timeout occurred while connecting to Agent DVR server"
			) from exception
		except (
			aiohttp.ClientError,
			aiohttp.ClientResponseError,
			socket.gaierror,
		) as exception:
			self.disconnected = datetime.now()
			self._conFailed = True
			
			raise AgentConnectionError(
				"Error occurred while communicating with Agent DVR server"
			) from exception

		content_type = response.headers.get("Content-Type", "")
		if "application/json" not in content_type:
			self.disconnected = datetime.now()
			self._conFailed = True

			text = await response.text()
			raise AgentError(
				"Unexpected response from the Agent DVR server",
				{"Content-Type": content_type, "response": text},
			)

		json = await response.json()
		if json.get('error') is not None:
			#error from Agent - Server is still available
			raise AgentError(
				"Error from the Agent DVR server",
				{"Content-Type": content_type, "response": json},
			)
		self._conFailed = False
		return json

	async def get_devices(self) -> List[Device]:
		"""Get a list of devices from the Agent API."""
		raw_objects = await self._agent_request('get', Agent.OBJECTS_URL)
		if raw_objects is not None:
			self.devices = []
			self.locations = raw_objects['locations'];
			for raw_result in raw_objects['objectList']:
				self.devices.append(Device(self, raw_result))
		return self.devices
	
	async def get_profiles(self) -> Optional[dict]:
		"""Get a list of Profiles from the Agent API."""
		raw_profiles = await self.get_state('command.cgi?cmd=getProfiles')
		if raw_profiles is not None:
			self.profiles = raw_profiles['profiles']

		return self.profiles

	async def get_active_profile(self) -> Optional[str]:
		"""Get the name of the active profile from the Agent API."""
		profiles = await self.get_profiles()
		if profiles is None:
			return None

		for profile in profiles:
			if profile['active']:
				return profile['name']
		return None

	async def set_active_profile(self, profile_name):
		"""Set the Agent profile to the given id."""
		await self.get_state('command.cgi?cmd=setProfileByName&name={0}'.format(profile_name))

	async def arm(self):
		"""Arm Agent."""
		await self.get_state('command.cgi?cmd=arm')

	async def disarm(self):
		"""Disarm Agent."""
		await self.get_state('command.cgi?cmd=disarm')

	async def update(self):
		state = await self.get_state('command.cgi?cmd=getStatus')
		if state is not None:
			self._raw_result = state

	@property
	def is_available(self) -> bool:
		"""Indicate if this Agent service is currently available."""
		return not self._conFailed

	@property
	def name(self) -> str:
		"""Return name of the server."""
		return self._raw_result["name"]

	@property
	def unique(self) -> str:
		"""Return unique identifier for the server."""
		return self._raw_result["unique"]

	@property
	def remote_access(self) -> bool:
		"""Return unique identifier for the server."""
		return self._raw_result["remoteAccess"]

	@property
	def version(self) -> str:
		"""Return version of the server."""
		return self._raw_result["version"]

	@property
	def raw_result(self) -> dict:
		return self._raw_result

	@property 
	def device_count(self) -> int:
		return self._raw_result["devices"]

	@property
	def is_armed(self) -> bool:
		"""Indicate if Agent is armed."""
		return self._raw_result["armed"]

	@staticmethod
	def _build_server_url(server_host) -> str:
		"""Build the server url making sure it ends in a trailing slash."""
		if server_host[-1] == '/':
			return server_host
		return '{}/'.format(server_host)



"""Exceptions for Agent DVR."""


class AgentError(Exception):
    """Generic Agent DVR exception."""

    pass


class AgentConnectionError(AgentError):
    """Agent DVR connection exception."""

    pass



import unittest
import aiohttp

from agent import a
from agent.device import TimePeriod


import asyncio

async def Test (url):
	tc = a.Agent(url)
	sess = aiohttp.ClientSession()
	tc2 = a.Agent(url,sess)

	await tc.update()
	print(tc.is_available)
	print(await tc.get_profiles())
	print(await tc.get_active_profile())
	print(tc.is_armed)
	print(tc.unique)
	await tc.arm()
	await tc.disarm()
	print(tc.name)
	print(tc.version)
	print(tc.raw_result)
	print(tc.remote_access)
	print(tc.device_count)

	m= await tc.get_devices()
	for o in m:
		print(o.name)
		print(o._client._server_url)
		print(o.id)
		print(o.typeID)
		print(o.mjpeg_image_url)
		print(o.still_image_url)
		print(o.mp4_url)
		print(o.webm_url)
		print(o.recording)
		print(o.location)
		print(o.width)
		print(o.height)
		print(o.mjpegStreamWidth)
		print(o.mjpegStreamHeight)
		print(o.alerted)
		print(o.online)
		print(o.detector_active)
		print(o.alerts_active)
		print(o.connected)
		print(o.has_ptz)
		print(o.raw_result)
		await o.snapshot()
		await o.alerts_on()
		await o.alerts_off()
		print(await o.get_events(TimePeriod.HOUR))
		await o.update()

	print(await tc.get_profiles())
	await tc.set_active_profile('home')
	print(await tc.get_active_profile())
	await tc.set_active_profile('away')
	print(await tc.get_active_profile())

	await tc.close()
	await tc2.close()
	await sess.close()

	print("end")

loop = asyncio.get_event_loop()
loop.run_until_complete(Test("http://localhost:8090/"))
